/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2006 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2009      by GJ Designing Center Print Business Group
 *                             RICOH COMPANY,LTD. JAPAN
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: bluetooth.c,v 1.1 2009/12/26 09:26:42 roi Exp $
 */

/*****************************************************************************
 * FILE: ecrobot_bluetooth.c
 *
 * COPYRIGHT 2008 Takashi Chikamasa <takashic@cybernet.co.jp>
 *
 * <About leJOS NXJ>
 *  leJOS NXJ is a full firmware replacement of LEGO Mindstorms NXT and 
 *  designed for Java programming environment for the NXT 
 *  ( For more detailed information, please see: http://lejos.sourceforge.net/ )
 *  In the leJOS NXJ distribution, C source files for NXT platform layer is also
 *  included besides with the Java VM. The platform C source code is well
 *  structured, comprehensive, and achieved higher performance than the LEGO's
 *  one. Therefore, leJOS NXJ (platform) is also the best GCC based C/C++  
 *  development platform for NXT.
 *
 *  The contents of this file are subject to the Mozilla Public License
 *  Version 1.0 (the "License"); you may not use this file except in
 *  compliance with the License. You may obtain a copy of the License at
 *  http://www.mozilla.org/MPL/
 *
 *  Software distributed under the License is distributed on an "AS IS"
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *  License for the specific language governing rights and limitations
 *  under the License.
 *
 *  The Original Code is TinyVM code, first released March 6, 2000,
 *  later released as leJOS on September 23, 2000.
 *
 *  The Initial Developer of the Original Code is Jose H. Solorzano.
 *
 *  Contributor(s): see leJOS NXJ ACKNOWLEDGEMENTS .
 *
 * <About TOPPERS OSEK>
 *  TOPPERS OSEK is an open source OSEK kernel and developed by TOPPERS project.
 *  TOPPERS(Toyohashi OPen Platform for Embedded Real-time Systems) has been managed 
 *  by a Non Profit Organization founded in Sep. 2003 and has been led by Professor
 *  Hiroaki Takada of Nagoya University in Japan. 
 *
 *  TOPPERS OSEK program is covered by the TOPPERS License as published
 *  by the TOPPERS PROJECT (http://www.toppers.jp/en/index.html).
 *
 *****************************************************************************/

/*
 *	֥롼ġ󥿥եɥ饤
 */

#include <kernel.h>
#include <t_syslog.h>
#include "target_config.h"
#include "target_syssvc.h"
#include "target_serial.h"
#include "logtask.h"
#include "kernel_cfg.h"

#define BT_BUF_SIZE 254 /* maximum size of Bluetooth Tx/Rx buffer in byte */

#define MSG_ACCEPT_CONNECTION 9
#define MSG_PIN_CODE 10
#define MSG_OPEN_STREAM 11
#define MSG_CONNECT_RESULT 19
#define MSG_REQUEST_PIN_CODE 21
#define MSG_REQUEST_CONNECTION 22

#define MSG_GET_LOCAL_ADDR 39
#define MSG_GET_LOCAL_ADDR_RESULT 40

#ifndef CONSOLE_PORTID
#define CONSOLE_PORTID     LOGTASK_PORTID
#endif

static volatile uint8_t handle;
static uint8_t receiveBuf[BT_BUF_SIZE];

const uint8_t bluetooth_pincode[] = BLUETOOTH_PINCODE ;
uint8_t bluetooth_device_address[8];

/**
 * send Bluetooth command
 *
 * @param cmd: Bluetooth command
 * @param buflen: length of the command to send
 */
static void send_bt_command(uint8_t *cmd, int_t bufLen)
{
	int checkSum = 0;
	int i;

	SIOPACKET *p;
	uint8_t   *buf;
	int       len;

	bluetooth_get_packet(CONSOLE_PORTID, &p, &len);
	buf = &p->buf[0];
	buf[0] = (uint8_t) (bufLen + 2);

	for(i = 0; i < bufLen; i++)
	{
		buf[i+1] = cmd[i];
		checkSum += cmd[i];
	}

    checkSum = -checkSum;
    buf[bufLen+2] = (uint8_t)((checkSum >> 8) & 0xff);
    buf[bufLen+3] = (uint8_t)(checkSum & 0xff);
	p->len = bufLen+3;

	bluetooth_send(CONSOLE_PORTID, p);
}

/**
 * read Bluetooth command
 *
 * @param cmd: Bluetooth command
 * @param buflen: length of the command buffer
 * @return length of the return command
 */
static uint_t read_bt_command(uint8_t *cmd, uint_t bufLen)
{
	int_t  checkSum, negSum, i;
	uint_t len;

	memset(&receiveBuf[0],0,sizeof(receiveBuf));
	bluetooth_receive(CONSOLE_PORTID, &receiveBuf[0]);
	len = receiveBuf[0];
	cmd[0] = (uint8_t) len;

	if (len == 0)
		return 0;

	checkSum = len;

	if (len-1 <= bufLen)
	{
		for(i = 1; i < len-1; i++) 
		{
			cmd[i] = receiveBuf[i];
			checkSum += (cmd[i] & 0xff);
		}
		negSum = (receiveBuf[len-1] & 0xff) << 8;
		negSum += (receiveBuf[len] & 0xff);
		if (checkSum + negSum == 65536)
		{
			 return len-1;
		}
		else
		{
			return 0;
		}
	}
	return 0;
}

/**
 * get Bluetooth local device address
 *
 * @param bd_addr: buffer for getting Bluetooth Device Address
 *                 (7bytes hex array data)
 * @return: 1(succeeded)/0(failed)
 */
uint8_t get_bluetooth_device_address(uint8_t* bd_addr)
{
	int i, count = 0;
	uint8_t msg[1];
	uint8_t reply[32];

	msg[0] = MSG_GET_LOCAL_ADDR;
	send_bt_command(msg,1);

	while(count < 3000)
	{
		read_bt_command(reply,32);

		if (reply[0] != 0 && reply[1] == MSG_GET_LOCAL_ADDR_RESULT)
		{
			for(i=0;i<7;i++)
			{
				bd_addr[i] = reply[i+2];
			}
			return 1;
		}
		dly_tsk(10);
		count += 10;
	}
	return 0;
}

/**
 * open a Bluetooth stream with the paired device
 */
static void open_bluetooth_stream(void)
{
	uint8_t msg[2];

	if (bluetooth_get_connect_state(CONSOLE_PORTID) == BLUETOOTH_CONNECTED)
	{
		msg[0] = MSG_OPEN_STREAM;
		msg[1] = handle;
		send_bt_command(msg, 2);
		dly_tsk(100);

		bluetooth_set_arm7_cmd(); /* set stream mode */
		bluetooth_set_connect_state(CONSOLE_PORTID, BLUETOOTH_STREAM);
	}
}

/**
 * connect the slave device to a master device
 * NOTE that this API is assumed to be invoked in a loop to establish a connection
 *
 * @param pin: pin code (8 to 16 characters)
 * @return: Bluetooth device status (BLUETOOTH_NO_INIT/BLUETOOTH_INITIALIZED/BLUETOOTH_CONNECTED/BLUETOOTH_STREAM)
 */
static int_t connect_bluetooth_slave(const int8_t *pin)
{
	int bt_status = bluetooth_get_connect_state(CONSOLE_PORTID);
	int i;
	uint8_t  msg[32];
	uint8_t  reply[32];
	uint8_t  dummy[32];
	uint8_t  bd_addr[7];

	if (bt_status == BLUETOOTH_NO_INIT)
	{

		if (get_bluetooth_device_address(bluetooth_device_address))
		{
			bt_status = BLUETOOTH_INITIALIZED;
		}
		else
		{
			bt_status = BLUETOOTH_CONNECT_ERROR;
		}
	}
	else if (bt_status == BLUETOOTH_INITIALIZED)
	{
		/* wait for a master device
		 * (based on LEJOS Bluetooth.waitForConnection)
    	 */
		read_bt_command(reply, 32);

		if (reply[0] != 0)
		{
			if (reply[1] == MSG_REQUEST_PIN_CODE)
			{
				for(i = 0; i < 7; i++)
				{
					bd_addr[i] = reply[i+2];
				}

				memset(msg,0,sizeof(msg));
				msg[0] = MSG_PIN_CODE;
				/* fill in slave bd_addr */
				for(i = 0; i < 7; i++)
				{
					msg[i+1] = bd_addr[i];
				}
				/* fill in pin code */
				for(i = 0; i<16 && pin[i] != '\0'; i++)
				{
					msg[i+8] = (uint8_t)pin[i];
				}
				send_bt_command(msg, 24);
			}
			else if (reply[1] == MSG_REQUEST_CONNECTION)
			{
				for(i = 0; i < 7; i++)
				{
					bd_addr[i] = reply[i+2];
				}
				msg[0] = MSG_ACCEPT_CONNECTION;
				msg[1] = 1;
				send_bt_command(msg, 2);
			}
			else if (reply[1] == MSG_CONNECT_RESULT)
			{
				dly_tsk(300);
				read_bt_command(dummy,32);

				if (dummy[0] == 0)
				{
					handle = reply[3];
					bt_status = BLUETOOTH_CONNECTED;
				}
			}
		}
	}
	bluetooth_set_connect_state(CONSOLE_PORTID, bt_status);
	return bt_status;
}

/**
 * initialize NXT as a Bluetooth slave device
 * NOTE that this API is assumed to be invoked in a loop to establish a connection
 *
 * @param pin: pin code (8 to 16 characters)
 */
void init_bluetooth_slave(const int8_t *pin)
{
	connect_bluetooth_slave(pin);
	open_bluetooth_stream();
}

bool_t bluetooth_connect(void)
{
	do{
		init_bluetooth_slave(bluetooth_pincode);
		dly_tsk(40);
	}while(bluetooth_get_connect_state(CONSOLE_PORTID) < BLUETOOTH_STREAM);
	return true;
}

uint8_t *bluetooth_get_pincode(void)
{
	return bluetooth_pincode;
}

uint8_t *bluetooth_get_device_address(void)
{
	return bluetooth_device_address;
}

#if 0	/* ROI DEBUG */
void bluetooth_initialize(intptr_t exinf)
{
	bluetooth_set_connect_function(CONSOLE_PORTID, bluetooth_connect);
}
#endif	/* ROI DEBUG */

