/*
 *  TOPPERS Automotive Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *      Automotive Kernel
 * 
 *  Copyright (C) 2006 by Witz Corporation, JAPAN
 *  Copyright (C) 2010      by GJ Designing Center Print Business Group
 *                             RICOH COMPANY,LTD. JAPAN
 * 
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 * 
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: sys_config.c,v 1.1 2010/02/13 18:14:24 roi Exp $
 */

/*
 *	åȥƥ¸⥸塼LEGO MINDSTORMS NXTѡ
 */

#include	"osek_kernel.h"

/*interrupt mask*/
unsigned long int_mode_table[MAX_INT_NUM]={
    0,
    SMR_SYSIRQ,
    SMR_PIOA,
    0,
    0,
    SMR_SPI,
    SMR_US0,
    SMR_US1,
    SMR_SSC,
    SMR_TWI,
    SMR_PMW,
    0,
    SMR_TC0,
    SMR_TC1,
    SMR_TC2,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0
};

/* Default handlers for the three general kinds of interrupts that the
 * ARM core has to handle. These are defined in irq.s, and just freeze
 * the board in an infinite loop.
 */
extern void default_isr(void);
extern void default_fiq(void);
extern void spurious_isr(void);
extern void nxt_toppers_stop(void);

/*
 *  åȥƥ¸ν
 */
void
sys_initialize(void)
{
	int i;

   /* If we're coming from a warm boot, the AIC may be in a weird
    * state. Do some cleaning up to bring the AIC back into a known
    * state:
    *  - All interrupt lines disabled,
    *  - No interrupt lines handled by the FIQ handler,
    *  - No pending interrupts,
    *  - AIC idle, not handling an interrupt.
    */
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_IDCR), 0xFFFFFFFF);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_FFDR), 0xFFFFFFFF);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_ICCR), 0xFFFFFFFF);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_EOICR), 1);

   /* Enable debug protection. This is necessary for JTAG debugging, so
    * that the hardware debugger can read AIC registers without
    * triggering side-effects.
    */
	*((volatile unsigned long *)(TADR_AIC_BASE+TOFF_AIC_DCR)) =  1;

   /* Set default handlers for all interrupt lines. */
	for (i = 0 ; i < MAX_INT_NUM ; i++) {
		if(_vector_table[i] != 0){
			sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_IDCR), 1 << i);
			sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SMR+i*4), int_mode_table[i]);
			sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SVR+i*4), _vector_table[i]);
			sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_IECR), 1 << i);
		}
		else{
			sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SMR+i*4), 0);
			sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SVR+i*4), (unsigned long)default_isr);
		}
	}
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SVR+IRQ_FIQ_PID*4), (unsigned long)default_fiq);
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_SVR+TOFF_AIC_SPU), (unsigned long)spurious_isr);
}


/*
 *  åȥƥνλ
 */
void
sys_exit(void)
{
	sil_wrw_mem((VP)(TADR_AIC_BASE+TOFF_AIC_IDCR), 0xFFFFFFFF);
    nxt_toppers_stop();
    while(1);
}


